"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.navRunnerStep = exports.Runner = void 0;
const Logger_1 = require("./helpers/Logger");
const AppHelper_1 = require("./helpers/AppHelper");
const ErrorHandler_1 = require("./helpers/ErrorHandler");
const ResultManager_1 = require("./helpers/ResultManager");
const setup_1 = require("../../../setup/setup");
const screenshotManager_1 = require("../../utils/screenshotManager");
class Runner {
    constructor(page) {
        this.page = page;
        this.logger = new Logger_1.Logger();
        this.appHelper = new AppHelper_1.AppHelper(this.page, this.logger);
        this.errorHandler = new ErrorHandler_1.ErrorHandler(this.page);
        this.resultManager = new ResultManager_1.ResultManager(this.logger);
    }
    run() {
        return __awaiter(this, void 0, void 0, function* () {
            let next = yield this.getNextNode({});
            for (; next; next = yield this.getNextNode(next)) {
                if (this.resultManager.isFailed())
                    break;
                const error = yield this.checkNode(next);
                if (error) {
                    screenshotManager_1.ScreenshotManager.saveFailScreenshot(this.page.cucumberPage, `-${this.resultManager.errorsCount()}`);
                }
            }
            this.resultManager.showResult();
        });
    }
    getNextNode(path) {
        return this.withTimeLimit({
            promise: this.appHelper.getNextNode(path),
            expiredTimeoutAction: this.logger.findNextNode(path)
        }).catch(() => null);
    }
    checkNode(path) {
        return __awaiter(this, void 0, void 0, function* () {
            let error = yield this.handleError({
                action: () => __awaiter(this, void 0, void 0, function* () {
                    yield this.appHelper.openNode(path);
                }),
                onError: (errorMessage) => {
                    this.logger.showError(this.logger.openNodeError(path), errorMessage, this.logger.level.openNode);
                    this.resultManager.addError({
                        definition: this.logger.openNodeError(path),
                        message: errorMessage
                    });
                },
                timeoutExpired: {
                    actionDescription: this.logger.openNode(path)
                }
            });
            if (error)
                return error;
            error = yield this.handleError({
                action: () => __awaiter(this, void 0, void 0, function* () {
                    yield this.appHelper.tryOpenForm(path);
                }),
                onError: (errorMessage) => {
                    this.logger.showError(this.logger.openFormError(path), errorMessage, this.logger.level.openForm);
                    this.resultManager.addError({
                        definition: this.logger.openFormError(path),
                        message: errorMessage
                    });
                },
                timeoutExpired: {
                    actionDescription: this.logger.openForm(path)
                }
            });
            return error;
        });
    }
    handleError(options) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.withTimeLimit({
                promise: this.errorHandler.handle(options.action, options.onError),
                expiredTimeoutAction: options.timeoutExpired.actionDescription
            }).catch(() => true);
        });
    }
    // Ограничиваем промисы таймаутом, так как у шага нет ограничения по времени, а узел или форма могут открываться слишком долго
    withTimeLimit(options) {
        return __awaiter(this, void 0, void 0, function* () {
            return yield new Promise((res, rej) => {
                this.timer = setTimeout(() => {
                    rej();
                    this.resultManager.failForce({
                        definition: options.expiredTimeoutAction,
                        message: this.logger.expiredMessage()
                    });
                }, setup_1.config.largeTimeout * 2);
                options.promise.then(res).finally(() => {
                    clearTimeout(this.timer);
                });
            });
        });
    }
}
exports.Runner = Runner;
/**
 * Только с модулем mdt.apps
 */
function navRunnerStep() {
    return __awaiter(this, void 0, void 0, function* () {
        const runner = new Runner(this);
        yield runner.run();
    });
}
exports.navRunnerStep = navRunnerStep;
