"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.validateModuleSettings = void 0;
const types_1 = require("../../../src/types/types");
const template = require("../template").default;
const WRONG_SETTINGS_MESSAGE = {
    wrongFormat: `Неверный формат настроек, пожалуйста, укажите в формате \n${template}`,
    testsInvalid: `Неверный формат настроек тестов, пожалуйста, укажите в формате:\n${template.tests}`,
    appInvalid: `Неверный формат настроек данных для входа, пожалуйста, укажите в формате:\n${template.app}`,
    setupInvalid: `Неверный формат настроек тестового окружения, пожалуйста, укажите в формате:\n${template.setup}`,
    debugInvalid: `Неверный формат настроек отладки, пожалуйста, укажите в формате:\n${template.debug}`,
    debugModeInvalid: `Режим отладки может только следющие типы: ${types_1.debugMods.join(", ")}`,
    propertyMustBeBoolean: (prop) => `Свойство ${prop} может быть только булевого типа: true или false`
};
function validateModuleSettings(settings) {
    if (typeof settings !== "object" || Array.isArray(settings) || settings === null)
        throw WRONG_SETTINGS_MESSAGE.wrongFormat;
    if (!isTestsValid(settings))
        throw WRONG_SETTINGS_MESSAGE.testsInvalid;
    if (!isAppValid(settings))
        throw WRONG_SETTINGS_MESSAGE.appInvalid;
    if (!isSetupValid(settings))
        throw WRONG_SETTINGS_MESSAGE.setupInvalid;
    validateDebug(settings);
}
exports.validateModuleSettings = validateModuleSettings;
function isTestsValid(settings) {
    var _a, _b;
    if (!settings.tests)
        return false;
    const tests = settings.tests;
    if (!((_a = tests.features) === null || _a === void 0 ? void 0 : _a.length))
        return false;
    if (!((_b = tests.steps) === null || _b === void 0 ? void 0 : _b.length))
        return false;
    if (typeof tests.exitOnEnd != "boolean")
        return false;
}
function isAppValid(settings) {
    if (!settings.app)
        return false;
    const app = settings.app;
    if (typeof app.url != "string")
        return false;
    if (app.defaultUser && typeof app.defaultUser != "string")
        return false;
    return isUsersValid(app);
}
function isUsersValid(app) {
    if (!app.users)
        return false;
    if (Object.keys(app.users).length < 1)
        return false;
    const wrongUser = Object.values(app.users).some((user) => {
        return !(user === null || user === void 0 ? void 0 : user.login) || !user.password;
    });
    return !wrongUser;
}
function isSetupValid(settings) {
    if (!settings.setup)
        return false;
    const setup = settings.setup;
    if (setup.timeout !== void 0 && typeof setup.timeout != "number")
        return false;
    if (setup.slowMo !== void 0 && typeof setup.slowMo != "number")
        return false;
    if (setup.maxRetries !== void 0 && typeof setup.maxRetries != "number")
        return false;
    if (setup.reloadOnError !== void 0 && typeof setup.reloadOnError != "boolean")
        return false;
    if (setup.timezone !== void 0 && typeof setup.timezone != "string")
        return false;
}
function validateDebug(settings) {
    if (settings.debug) {
        const debug = settings.debug;
        if (!!debug.mode && !types_1.debugMods.includes(debug.mode))
            throw WRONG_SETTINGS_MESSAGE.debugModeInvalid;
        if (debug.devTools != void 0 && typeof debug.devTools != "boolean")
            throw WRONG_SETTINGS_MESSAGE.propertyMustBeBoolean("debug.devTools");
    }
}
